from flaskunittest import FlaskTestCase

import sorter
from datetime import datetime, date, time

class SorterTestCase(FlaskTestCase):
	def test_split_filename(self):
		testdata = [
				{'filename': 'asdasd', 'chunks': ['asdasd']},
				{'filename': 'a-b-c-d', 'chunks': ['a', 'b', 'c', 'd']},
				{'filename': 'a_', 'chunks': ['a', '']},
				{'filename': 'a-', 'chunks': ['a', '']},
				{'filename': 'a ', 'chunks': ['a', '']},
				{'filename': '-ß', 'chunks': ['', 'ß']},
				{'filename': 'b-a.mp4', 'chunks': ['b', 'a.mp4']},
				{'filename': '', 'chunks': ['']}
				]
		for test in testdata:
			result = sorter.split_filename(test['filename'])
			assert result == test['chunks'], 'result was {}, should be {}'.format(result, test)

	def test_extract_format_keyword_from_filename(self):
		testdata = [
				{'chunks': ['',''], 'format': ''},
				{'chunks': ['asd','720p'], 'format': '720p'},
				{'chunks': ['asd','720P'], 'format': '720p'},
				{'chunks': ['asd','#\ää'], 'format': '#\ää'},
				{'chunks': ['123'], 'format': '123'},
				]
		for test in testdata:
			result = sorter.extract_format_keyword_from_filename(test['chunks'])
			assert result == test['format'], 'result was {}, should be {}'.format(result, test)

	def test_filter_formats_by_filename(self):
		testdata = [
				{'chunks': ['ääüp','ßääää'], 'format': 0},
				{'chunks': ['123üß','720P'], 'format': 5},
				{'chunks': ['testvideo','1080p'], 'format': 4},
				{'chunks': ['mp3'], 'format': 7},
				]
		with self.requestContext:
			for test in testdata:
				result = sorter.filter_formats_by_filename(test['chunks'])
				assert result == test['formatid'], 'result was {}, should be {}'.format(result, test)
	
	def test_parse_filename(self):
		testdata = [
				{'filename': '', 'data': {'keywords': ['']}},
				{'filename': '18ss-mc-180413_720p.mp4', 'data': {'keywords': ['18ss', 'mc', '720p'], 'date': date(year=2018, month=4, day=13)}},
				{'filename': 'astaintern-astawiki-1080p.mp4', 'data': {'keywords': ['astaintern', 'astawiki', '1080p']}},
				{'filename': '15ss-zkk-extremale-codes-720p.mp4', 'data': {'keywords': ['15ss', 'zkk', 'extremale', 'codes', '720p']}},
				# TODO: missing test data for time
				]
		for test in testdata:
			result = sorter.parse_filename(sorter.split_filename(test['filename']))
			assert result == test['data'], 'result was {}, should be {}'.format(result, test['data'])

	def test_filter_lectures_by_keywords(self):
		testdata = [
				{'lectures': [], 'keywords': []},
				{'lectures': [{'title': 'a', 'result': True}, {'title': 'b:', 'result': False}], 'keywords': ['a']},
				{'lectures': [{'speaker': 'aca', 'result': True}, {'comment': 'bbd:', 'result': False}], 'keywords': ['c']},
				{'lectures': [{'internal': 'apäöa', 'result': False}, {'comment': 'bbd:', 'result': False}], 'keywords': ['c']},
				{'lectures': [{'internal': 'alll', 'result': False}, {'comment': 'bbdäo', 'result': True}], 'keywords': ['ä']},
				]
		for test in testdata:
			result = sorter.filter_lectures_by_keywords(test['lectures'], test['keywords'])
			for i in result:
				assert i.get('result')
			for i in test.get('lectures', []):
				assert (not i.get('result')) or (i in result)

	def test_filter_lectures_by_datetime(self):
		testdata = [
				{'lectures': [], 'date': None, 'time': None},
				{'lectures': [{'time': datetime(year=2000, month=1, day=1), 'result': True}], 'date': None, 'time': time(hour=0, minute=0)},
				{'lectures': [{'time': datetime(year=2000, month=1, day=1), 'result': False}], 'date': None, 'time': time(hour=0, minute=1)},
				{'lectures': [{'result': False}], 'date': None, 'time': time(hour=0, minute=1)},
				# TODO: add more testdata
				]
		for test in testdata:
			result = sorter.filter_lectures_by_datetime(test['lectures'], test.get('date'), test.get('time'))
			for i in result:
				assert i.get('result')
			for i in test.get('lectures', []):
				assert (not i.get('result')) or (i in result)
		pass
	def test_filter_formats_by_filename(self):
		pass
	def test_sort_file(self):
		testdata = [
				{'filename': '08ws-swt-081118.mp4', 'match': [104], 'fmt': 0},
				{'filename': '15ss-zkk-extremale-codes-1080p.mp4', 'match': [6095], 'fmt': 4},
				{'filename': '15ws-afi-151027-720p.mp4', 'match': [6326], 'fmt': 5},
				]
		with self.requestContext:
			for test in testdata:
				match, fmt = sorter.sort_file(test['filename'])
				assert len(match) == len(test['match'])
				for i in match:
					assert i['id'] in test['match'], '{} is not supposed to match, only {} is'.format(i['id'], test['match'])
				assert fmt == test['fmt'], 'format id {} is wronge, it is supposed to be {}'.format(fmt, test['fmt'])
